// ======================================================================
//
// HardPoints.cpp
// jeff grills
//
// copyright 1996 origin systems inc
//
// ======================================================================

#include "First.h"
#include "HardPoints.h"

#include "Gl.h"
#include "Iff.h"

// ======================================================================

HardPoints::Entry::Entry(void)
: xform(), name(NULL)
{
}

// ----------------------------------------------------------------------

HardPoints::Entry::~Entry(void)
{
	delete [] name;
	name = NULL;
}

// ======================================================================
	
HardPoints::HardPoints(Iff &iff, const char *id)
{
	int i;

	UNREF(id);

	users = 0;

	// allocate the hardpoint list
	count = iff.blockCount();
	entry = NEW Entry[count];

	// load all the hard points
	for (i = 0; i < count; i++)
	{
		Xform xform;

		iff.enterChunk();
		iff.read_floatXform(1, &xform);
		entry[i].setXform(xform);
		entry[i].setName(MEM_OWN(iff.read_string()));

#ifndef NDEBUG
		int bad;
		Vector x, y, z, c;

		entry[i].getXform().frameI(&x);
		entry[i].getXform().frameJ(&y);
		entry[i].getXform().frameK(&z);

		bad = false;
		c.cross(x, y);
		if (abs(c.dot(z) - CONST_REAL(1)) > CONST_REAL(0.05))
			bad |= 1;

		c.cross(z, x);
		if (abs(c.dot(y) - CONST_REAL(1)) > CONST_REAL(0.05))
			bad |= 2;

		c.cross(y, z);
		if (abs(c.dot(x) - CONST_REAL(1)) > CONST_REAL(0.05))
			bad |= 4;

		WARNING(bad, ("Hardpoints:: bad %d=index %s=name %s=mesh %d=code\n", i, entry[i].getName(), id, bad));
#endif

		iff.exitChunk();
	}
}

// ----------------------------------------------------------------------

HardPoints::~HardPoints(void)
{
	delete [] entry;
	entry = NULL;
}

// ----------------------------------------------------------------------

void HardPoints::render(void) const
{
	int    i;
	Vector v[4];

	for (i = 0; i < count; i++)
	{
		DEBUG_FATAL(!entry, ("HardPoints::render !entry"));

		// get the center
		entry[i].getXform().getPosition(v+0);

		// get axis vectors
		entry[i].getXform().frameI(v+1);
		entry[i].getXform().frameJ(v+2);
		entry[i].getXform().frameK(v+3);

		// construct points from the axis
		v[1] += v[0];
		v[2] += v[0];
		v[3] += v[0];

		Gl::setMaterial(MaterialList::red());
		Gl::line(v[0], v[1]);

		Gl::setMaterial(MaterialList::green());
		Gl::line(v[0], v[2]);

		Gl::setMaterial(MaterialList::blue());
		Gl::line(v[0], v[3]);
	}
}

// ----------------------------------------------------------------------

bool HardPoints::findName(const char *name, int *index) const
{
	int i;

	for (i = 0; i < count; i++)
		if (strcmp(name, entry[i].getName()) == 0)
		{
			*index = i;
			return true;
		}

	return false;
}

// ======================================================================
